;;
;; mailman.scm
;;
;; Mailman management interface - public API
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit mailman))

(module
 mailman
 (
  mailman-list-name
  mailman-list-members

  find-mailman-list
  
  email-in-mailman-list?

  mailman-ensure-member
  mailman-ensure-not-member
  
  mailman-compare-members
  
  mailman-sync-members
  )

 (import scheme
	 (chicken base)
	 (chicken module)
	 mailman2
	 mailman-common
	 util-bst-lset
	 configuration
	 mailman3)

 ;; Syntax for simplifying export of case-version procedures
 (define-syntax define-mailman-proc
   (syntax-rules ()
     ((_ name proc2)
      (begin
	(export name)
	(define (name . args)
	  (case (*mailman-version*)
	    ((2) (apply proc2 args))))))
     ((_ name proc2 proc3)
      (begin
	(export name)
	(define (name . args)
	  (case (*mailman-version*)
	    ((2) (apply proc2 args))
	    ((3) (apply proc3 args))))))))

 (define-mailman-proc list-mailman-lists
   list-mailman2-lists list-mailman3-lists)
 (define-mailman-proc list-mailman-list-members list-mailman2-list-members)
 (define-mailman-proc load-mailman-list load-mailman2-list)
 (define-mailman-proc load-mailman-lists load-mailman2-lists)

 ;; List of lists, returns the whole list record (including name)
 (define (find-mailman-list lsts name)
   (assoc name lsts))

 (define-mailman-proc add-email-to-mailman-list add-email-to-mailman2-list)
 (define-mailman-proc remove-email-from-mailman-list remove-email-from-mailman2-list)

 ;; Ensures given email is in given ML
 (define (mailman-ensure-member ml email)
   (when (not (email-in-mailman-list? ml email))
     (add-email-to-mailman-list (mailman-list-name ml) email)))

 ;; Makes sure given member is removed
 (define (mailman-ensure-not-member ml email)
   (when (email-in-mailman-list? ml email)
     (remove-email-from-mailman-list (mailman-list-name ml) email)))


 ;; Returns two values - missing members and surplus list members
 (define (mailman-compare-members ml emails)
   (let* ((mlemails (list->lset (mailman-list-members ml)))
	  (emails (list->lset emails))
	  (surplus (lset-subtract mlemails emails))
	  (missing (lset-subtract emails mlemails)))
     (values (lset->list missing)
	     (lset->list surplus))))

 ;; Ensures given ML subscribers are exactly what is in emails list
 (define (mailman-sync-members ml emails)
   (let-values (((missing surplus)
		 (mailman-compare-members ml emails)))
     (let ((listname (mailman-list-name ml)))
       (let loop ((emails missing))
	 (when (not (null? emails))
	   (add-email-to-mailman-list listname (car emails))
	   (loop (cdr emails))))
       (let loop ((emails surplus))
	 (when (not (null? emails))
	   (remove-email-from-mailman-list listname (car emails))
	   (loop (cdr emails)))))))

 )
